#include "Game.h"

Game::Game()
{

}

Game::~Game()
{

}

void Game::Init(const char* title, int xpos, int ypos, int width, int height, bool fullscreen)
{
	int flags = 0;
	if (fullscreen) flags = SDL_WINDOW_FULLSCREEN;

	if (SDL_Init(SDL_INIT_EVERYTHING) == 0)
	{
		window = SDL_CreateWindow(title, xpos, ypos, width, height, flags);
		renderer = SDL_CreateRenderer(window, -1, 0);
		isRunning = true;
	}
	else
	{
		isRunning = false;
	}
}

SDL_Texture* Game::LoadTexture(const char* path, bool doOutput)
{

	SDL_Surface* tmpSurface = IMG_Load(path);
	SDL_Texture* tmpTexture = SDL_CreateTextureFromSurface(renderer, tmpSurface);
	SDL_FreeSurface(tmpSurface);
	if (tmpTexture != NULL) 
	{
		return tmpTexture; 
	}

	return NULL;
}

void Game::RenderText(TTF_Font* font, SDL_Color color, const char *text, int x, int y, int padding) // 0 - left // 1 - center // 2 - right
{
	SDL_Surface* textSurface = TTF_RenderText_Solid(font, text, color);
	SDL_Texture* texta = SDL_CreateTextureFromSurface(renderer, textSurface);
	int text_width = textSurface->w;
	int text_height = textSurface->h;
	SDL_FreeSurface(textSurface);
	if (padding == 1)
	{
		x = x - text_width / 2;
	}
	else if(padding == 2)
	{
		x = x - text_width;
	}
	SDL_Rect renderQuad = { x, y, text_width, text_height };
	SDL_RenderCopy(renderer, texta, NULL, &renderQuad);
	SDL_DestroyTexture(texta);
}

void Game::RenderRzufText(TTF_Font* font, const char* text, int x, int y, int padding, int scale) // 0 - left // 1 - center // 2 - right
{
	SDL_Color textColor = { 255, 255, 255, 0 };
	SDL_Color textColorB = { 0, 162, 232, 0 };
	SDL_Surface* textSurface = TTF_RenderText_Solid(font, text, textColor);
	SDL_Surface* textSurfaceB = TTF_RenderText_Solid(font, text, textColorB);
	SDL_Texture* textTexture = SDL_CreateTextureFromSurface(renderer, textSurface);
	SDL_Texture* textTextureB = SDL_CreateTextureFromSurface(renderer, textSurfaceB);
	int text_width = textSurface->w * scale;
	int text_height = textSurface->h * scale;
	SDL_FreeSurface(textSurface);
	if (padding == 1)
	{
		x = x - text_width / 2;
	}
	else if (padding == 2)
	{
		x = x - text_width;
	}
	SDL_Rect renderQuad = { x, y, text_width, text_height };
	SDL_Rect renderQuadB = { x + scale, y + scale, text_width, text_height };
	SDL_RenderCopy(renderer, textTexture, NULL, &renderQuad);
	SDL_RenderCopy(renderer, textTextureB, NULL, &renderQuadB);
	SDL_DestroyTexture(textTexture);
	SDL_DestroyTexture(textTextureB);
}

void Game::DrawTexture(SDL_Texture* textureToDraw, int x, int y, int scale, double rotation)
{
	int w, h;
	SDL_QueryTexture(textureToDraw, NULL, NULL, &w, &h);

	SDL_Rect tmpRect;
	tmpRect.h = h * scale;
	tmpRect.w = w * scale;
	tmpRect.x = x - tmpRect.w / 2;
	tmpRect.y = y - tmpRect.h / 2;

	SDL_RenderCopyEx(renderer, textureToDraw, NULL, &tmpRect, rotation, NULL, SDL_FLIP_NONE);
}

void Game::SetIcon(const char* path)
{
	SDL_Surface* icon = IMG_Load(path);
	SDL_SetWindowIcon(window, icon);
}

void Game::HandleEvents()
{
	SDL_Event event;
	SDL_PollEvent(&event);
	switch (event.type)
	{
	case SDL_QUIT:
		isRunning = false;
		break;
	}
}

void Game::Update()
{
	cnt++;
	SDL_GetMouseState(&mouseX, &mouseY);
	if (SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(SDL_BUTTON_LEFT))  mouseL = true; else mouseL = false;
	if (SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(SDL_BUTTON_RIGHT)) mouseR = true; else mouseR = false;

	int wW, wH;
	SDL_GetWindowSize(window, &wW, &wH);
	if (this->windowHeight != wH || this->windowWidth != wW)
	{
		this->wSizeChanged = true;
		this->windowHeight = wH;
		this->windowWidth = wW;
	}
	else this->wSizeChanged = false;
}

void Game::RenderStart()
{
	SDL_RenderClear(renderer);
	//rzeczy do renderowania przed wszystkimi obiektami here//
}

void Game::RenderFinish()
{
	//rzeczy do renderowania po wszystkich obiektach here//
	SDL_RenderPresent(renderer);
}

void Game::Clean()
{
	SDL_DestroyWindow(window);
	SDL_DestroyRenderer(renderer);
	SDL_Quit();
}