#include "MapManager.h"

MapManager::MapManager()
{

}

MapManager::~MapManager()
{

}

int MapManager::SaveMapToFile()
{
	int playerCount = 0;
	int boxCount = 0;
	int targetCount = 0;

	for (int y = 0; y < maps[selectedMap].height; y++)
	{
		for (int x = 0; x < maps[selectedMap].width; x++)
		{
			switch (maps[selectedMap].map[y][x])
			{
			case 'P':
				playerCount++;
				break;
			case 'B':
				boxCount++;
				break;
			case 'T':
				targetCount++;
				break;
			default:
				break;
			}
		}
	}

	if (playerCount == 0) return 10; //"Nie znaleziono miejsca startu!";
	if (playerCount > 1) return 11; // "Znaleziono wiecej juz 1 miejsce startu!";
	if (targetCount > boxCount) return 12; // "Liczba pozycji docelowych przewyzsza liczbe skrzyn!";
	if (targetCount == 0) return 13; // "Nie znaleziono zadnych skrzyn!";

	remove((maps[selectedMap].filename).c_str());
	ofstream savefile((maps[selectedMap].filename).c_str());
	if (savefile.is_open())
	{
		savefile << "name=Custom map\n";
		savefile << "width=" << to_string((int)maps[selectedMap].width) << "\n";
		savefile << "height=" << to_string((int)maps[selectedMap].height) << "\n";
		savefile << "tcount=" << to_string((int)targetCount) << "\n";
		savefile << "bcount=" << to_string((int)boxCount) << "\n";
		savefile << "difficulty=1\n";
		savefile << "difficultylevel=" << to_string((int)maps[selectedMap].difficultylevel) << "\n";
		savefile << "timelimit=60\nmapstart\n";
		for (int i = 0; i < maps[selectedMap].height; i++)
		{
			for (int j = 0; j < maps[selectedMap].width; j++)
				savefile << maps[selectedMap].map[i][j];
			savefile << "\n";
		}

		savefile << "mapend\neof\n";
		savefile.close();
	}

	maps[selectedMap] = LoadMap((maps[selectedMap].filename).c_str());
	maps[selectedMap].Reset();

	return 1;
}

int MapManager::GetRandomMapID(int difficultyLevel)
{
	ListMaps();
	int output = 0;

	do
	{
		output = rand() % this->mapsCount;
	} while (this->maps[output].difficultylevel != difficultyLevel || output == lastMap);

	lastMap = output;
	return output;
}

void MapManager::ListMaps(string path)
{
	mapsCount = 0;
	for (const auto& entry : std::filesystem::directory_iterator(path)) mapsCount++;

	maps = new Map[mapsCount];
	int i = 0;

	for (const auto& entry : std::filesystem::directory_iterator(path))
	{
		string path{ entry.path().u8string() };
		maps[i] = LoadMap(path);
		i++;
	}

	Map* sortBuffer = new Map[mapsCount];

	int currentMapID = 0;
	for (int i = 0; i < 20; i++) //dla tak prostego sortowania nie ma sensu implementowa niewiadomo jakich algorytmw
	{
		for (int m = 0; m < this->mapsCount; m++)
		{
			if (this->maps[m].difficulty == i)
			{
				sortBuffer[currentMapID] = this->maps[m];
				currentMapID++;
			}
		}
	}

	this->maps = sortBuffer;
}


bool MapManager::checkInFile(string name, string line, int* intoutput, string* stringoutput)
{
	name = name + "=";

	if (line.find(name) != string::npos)
	{
		line = line.erase(0, name.length());

		if (intoutput != nullptr) *intoutput = stoi(line);
		if (stringoutput != nullptr) *stringoutput = line;
		return true;
	}
	else
	{
		return false;
	}
}


Map MapManager::LoadMap(string name)
{
	Map newMap;
	newMap.filename = name;
	bool readmap = false;
	int currentbox = 0, currenttarget = 0, currenty = 0;
	string line;
	ifstream mapfile(name);
	if (mapfile.is_open())
	{
		while (getline(mapfile, line))
		{
			if (readmap)
			{
				if (line == "mapend")
				{
					readmap = false;
				}
				else
				{
					for (int i = 0; i < newMap.width; i++)
					{

						newMap.map[currenty][i] = line[i];
						if (line[i] == 'P')
						{
							newMap.playerPositionDefault[0] = i;
							newMap.playerPositionDefault[1] = currenty;
						}

						if (line[i] == 'B')
						{
							newMap.boxPositionDefault[0][currentbox] = i;
							newMap.boxPositionDefault[1][currentbox] = currenty;
							currentbox++;
						}

						if (line[i] == 'T')
						{
							newMap.targetPosition[0][currenttarget] = i;
							newMap.targetPosition[1][currenttarget] = currenty;
							currenttarget++;
						}

					}
				}

				currenty++;
			}
			else
			{
				if (checkInFile("name", line, nullptr, &newMap.name)) {}
				else if (checkInFile("width", line, &newMap.width)) {}
				else if (checkInFile("height", line, &newMap.height)) {}
				else if (checkInFile("difficulty", line, &newMap.difficulty)) {}
				else if (checkInFile("difficultylevel", line, &newMap.difficultylevel)) {}
				else if (checkInFile("tcount", line, &newMap.targetCount)) {}
				else if (checkInFile("bcount", line, &newMap.boxCount)) {}
				else if (checkInFile("timelimit", line, &newMap.timeLimit)) {}
				else if (line == "mapstart")
				{
					newMap.InitMapSize();
					readmap = true;
				}
			}

		}
		mapfile.close();
	}

	return newMap;
}